(*
    A simple metronome - Configuration form
    Copyright (c) 1998-2003 Alessandro Scotti
    www.ascotti.org

    Note: this unit will write in the registry

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
unit Config;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,
  Midi;

const
  BaseRegistryNode = 'Software\AScottiOrg\Metronome';

type
  TformConfig = class(TForm)
    grpMidi: TGroupBox;
    cboMidiDevice: TComboBox;
    lblMidiDevice: TLabel;
    lblMidiSound: TLabel;
    cboMidiSound: TComboBox;
    grpSound: TGroupBox;
    optNoSound: TRadioButton;
    optSystemBeep: TRadioButton;
    optMidi: TRadioButton;
    btnCancel: TButton;
    btnOk: TButton;
    btnTest: TButton;
    procedure FormCreate(Sender: TObject);
    procedure btnTestClick(Sender: TObject);
    procedure btnOkClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    FMidiChannel: integer;
    FTicker: TMidiTick;
    function GetMidiChannel: integer;
    function GetMidiSound: integer;
    procedure LoadFromRegistry;
    procedure SaveToRegistry;
  public
    { Public declarations }
    property MidiChannel: integer read GetMidiChannel;
    property MidiSound: integer read GetMidiSound;
  end;

var
  formConfig: TformConfig;

implementation

uses MMSystem, Registry;

{$R *.DFM}

const
  MidiPercussions : array[1..47] of string = (
  '38 Acoustic Snare',
  '35 Acoustic Bass Drum',
  '36 Bass Drum 1',
  '69 Cabasa',
  '52 Chinese Cymbal',
  '75 Claves',
  '42 Closed High Hat',
  '56 Cowbell',
  '49 Crash Cymbal 1',
  '57 Crash Cymbal 2',
  '40 Electric snare',
  '39 Hand Clap',
  '50 High Tom',
  '60 High Bongo',
  '67 High Agogo',
  '65 High Timbale',
  '43 High Floor Tom',
  '76 High Wood Block',
  '48 High-Mid Tom',
  '74 Long Guiro',
  '72 Long Whistle',
  '68 Low Agogo',
  '61 Low Bongo',
  '64 Low Conga',
  '41 Low Floor Tom',
  '45 Low Tom',
  '66 Low Timbale',
  '77 Low Wood Block',
  '47 Low-Mid Tom',
  '70 Maracas',
  '78 Mute Cuica',
  '80 Mute Triangle',
  '62 Mute High Conga',
  '79 Open Cuica',
  '81 Open Triangle',
  '46 Open High Hat',
  '63 Open High Conga',
  '44 Pedal High Hat',
  '53 Ride Bell',
  '59 Ride Cymbal 2',
  '51 Ride Cymbal 1',
  '73 Short Guiro',
  '71 Short Whistle',
  '37 Side Stick',
  '55 Splash Cymbal',
  '54 Tambourine',
  '58 Vibraslap' );

procedure TformConfig.FormCreate(Sender: TObject);
var
  I: integer;
  N: integer;
  C: TMidiOutCaps;
begin
  FTicker := nil;
  // Fill list of MIDI percussive sounds
  cboMidiSound.Items.Clear;
  for I:=Low(MidiPercussions) to High(MidiPercussions) do begin
    cboMidiSound.Items.AddObject( Copy(MidiPercussions[I],4,64),
      TObject( StrToInt(Copy(MidiPercussions[I],1,2)) ) );
  end;
  cboMidiSound.ItemIndex := 5; // Claves
  // Fill list of MIDI devices
  cboMidiDevice.Items.Clear;
  N := midiOutGetNumDevs;
  for I:=0 to N-1 do begin
    midiOutGetDevCaps( I, @C, sizeof(C) );
    cboMidiDevice.Items.Add( StrPas( C.szPName ) );
  end;
  //
  if( cboMidiDevice.Items.Count = 0 ) then begin
    optMidi.Enabled := false;
    grpMidi.Enabled := false;
    lblMidiDevice.Enabled := false;
    lblMidiSound.Enabled := false;
  end
  else begin
    cboMidiDevice.ItemIndex := 0;
  end;
  FMidiChannel := defaultMidiChannel;
  // Read parameters from registry
  LoadFromRegistry;
end;

procedure TformConfig.btnTestClick(Sender: TObject);
begin
  FTicker.Free;
  FTicker := nil;
  if( optSystemBeep.Checked ) then begin
    Beep;
  end
  else if( optMidi.Checked ) then begin
    FTicker := TMidiTick.Create( cboMidiDevice.ItemIndex,
      MidiSound,
      MidiChannel );
    FTicker.Tick;
  end;
end;

const
  sMidiChannel = 'MidiChannel';
  sMidiDevice = 'MidiDevice';
  sMidiSound = 'MidiSound';
  sSoundType = 'SoundType';

procedure TformConfig.LoadFromRegistry;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    if( Reg.OpenKey( BaseRegistryNode, false ) ) then begin
      // Read the MIDI device
      cboMidiDevice.ItemIndex := Reg.ReadInteger( sMidiDevice );
      // Read the MIDI sound
      cboMidiSound.ItemIndex := Reg.ReadInteger( sMidiSound );
      // Read the sound type
      case Reg.ReadInteger( sSoundType ) of
      0 : optNoSound.Checked := true;
      1 : optSystemBeep.Checked := true;
      2 : optMidi.Checked := true;
      end;
      // Read the MIDI channel
      FMidiChannel := Reg.ReadInteger( sMidiChannel );
    end;
  finally
    Reg.Free;
  end;
end;

procedure TformConfig.SaveToRegistry;
var
  Reg: TRegistry;
  Snd: integer;
begin
  Reg := TRegistry.Create;
  try
    if( Reg.OpenKey( BaseRegistryNode, true ) ) then begin
      // Save MIDI info
      Reg.WriteInteger( sMidiChannel, MidiChannel );
      Reg.WriteInteger( sMidiDevice, cboMidiDevice.ItemIndex );
      Reg.WriteInteger( sMidiSound, cboMidiSound.ItemIndex );
      // Save sound type
      Snd := 2;
      if( optNoSound.Checked ) then
        Snd := 0
      else if( optSystemBeep.Checked ) then
        Snd := 1;
      Reg.WriteInteger( sSoundType, Snd );
    end;
  finally
    Reg.Free;
  end;
end;

procedure TformConfig.btnOkClick(Sender: TObject);
begin
  SaveToRegistry;
end;

function TformConfig.GetMidiSound;
begin
  Result := integer(cboMidiSound.Items.Objects[cboMidiSound.ItemIndex]);
end;

function TformConfig.GetMidiChannel: integer;
begin
  Result := FMidiChannel;
end;

procedure TformConfig.FormHide(Sender: TObject);
begin
  LoadFromRegistry;
  FTicker.Free;
  FTicker := nil;
end;

procedure TformConfig.FormDestroy(Sender: TObject);
begin
  FTicker.Free;
end;

end.
