(*
    A simple metronome - Main form
    Copyright (c) 1998-2003 Alessandro Scotti
    www.ascotti.org

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
unit Main;

interface

uses
  Clock,
  Midi,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ExtCtrls;

type
  TformMain = class(TForm)
    trbTime: TTrackBar;
    updTime: TUpDown;
    edtTime: TEdit;
    shpOne: TShape;
    shpTwo: TShape;
    btnStartStop: TButton;
    shpThree: TShape;
    shpFour: TShape;
    cboBeat: TComboBox;
    Label1: TLabel;
    Label2: TLabel;
    chkMute: TCheckBox;
    btnSetup: TButton;
    btnClose: TButton;
    procedure FormShow(Sender: TObject);
    procedure trbTimeChange(Sender: TObject);
    procedure edtTimeChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnStartStopClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cboBeatChange(Sender: TObject);
    procedure btnSetupClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure shpOneMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    Beat: integer;
    BeatShape: array[0..3] of TShape;
    Clock: TClock;
    Ticker: TMidiTick;
    Marker: integer;
    procedure OnClockTick(Sender: TObject);
    procedure SetTempo( N: integer );
  public
    { Public declarations }
  end;

var
  formMain: TformMain;

implementation

uses Config;

{$R *.DFM}

const
  MetronomeTicks : array[ 1..39 ] of integer = (
    // Largo
    40, 42, 44, 46, 48, 50, 52, 54, 56, 58,
    // Larghetto
    60, 63, 66,
    // Adagio
    69, 72, 76,
    // Andante
    80, 84, 88, 92, 96, 100, 104, 108,
    // Moderato
    112, 116, 120,
    // Allegro
    126, 132, 138, 144, 152, 160,
    // Presto
    168, 176, 184, 192, 200, 208
  );

function GetNameOfTempo( N: integer ): string;
begin
  case N of
    40..59: Result := 'Largo';
    60..68: Result := 'Larghetto';
    69..79: Result := 'Adagio';
    80..111: Result := 'Andante';
    112..125: Result := 'Moderato';
    126..167: Result := 'Allegro';
    168..208: Result := 'Presto';
    else Result := '';
  end;
end;

procedure TformMain.FormShow(Sender: TObject);
var
  I: integer;
begin
  for I:=Low(MetronomeTicks)+1 to High(MetronomeTicks)-1 do
    trbTime.SetTick( MetronomeTicks[I] );
end;

procedure TformMain.trbTimeChange(Sender: TObject);
begin
  updTime.Position := trbTime.Position;
  Clock.Ticks := trbTime.Position;
end;

procedure TformMain.SetTempo( N: integer );
begin
  if( N >= trbTime.Min )and( N <= trbTime.Max ) then begin
    trbTime.Position := N;
    updTime.Position := N;
    Clock.Ticks := N;
    trbTime.Hint := GetNameOfTempo( N );
  end;
end;

procedure TformMain.edtTimeChange(Sender: TObject);
begin
  SetTempo( updTime.Position );
end;

procedure TformMain.FormCreate(Sender: TObject);
begin
  Clock := TClock.Create( true );
  Ticker := nil;
  Marker := 0;
  Beat := 0;
  BeatShape[0] := shpOne;
  BeatShape[1] := shpTwo;
  BeatShape[2] := shpThree;
  BeatShape[3] := shpFour;
  updTime.Position := trbTime.Position;
  cboBeat.ItemIndex := 3;
  Clock.Ticks := trbTime.Position;
  Clock.OnNotify := OnClockTick;
end;

procedure TformMain.btnStartStopClick(Sender: TObject);
begin
  // Create MIDI "tick" if needed
  if( Ticker = nil ) then begin
    Ticker := TMidiTick.Create(
      formConfig.cboMidiDevice.ItemIndex,
      formConfig.MidiSound,
      formConfig.MidiChannel );
  end;
  // Toggle start/stop
  if( Clock.Suspended ) then begin
    btnStartStop.Caption := '&Stop';
    Clock.Resume;
  end
  else begin
    btnStartStop.Caption := '&Start';
    Clock.Suspend;
    // Clear indicators, reset beat index
    cboBeatChange( Sender );
  end;
end;

procedure TformMain.FormDestroy(Sender: TObject);
begin
  Clock.Suspend;
  Clock.OnNotify := nil;
  Clock.Delay := 1;
  Clock.Terminate;
  while( Clock.Suspended ) do Clock.Resume;
  Sleep( 150 );
  Ticker.Free;
end;

procedure TformMain.OnClockTick(Sender: TObject);
const
  LiteColor = clRed;
var
  MaxBeat: integer;
begin
  MaxBeat := cboBeat.ItemIndex;
  if( MaxBeat = 0 ) then begin
    if( BeatShape[Beat].Brush.Color = LiteColor ) then
      BeatShape[Beat].Brush.Color := clWhite
    else
      BeatShape[Beat].Brush.Color := LiteColor;
  end
  else begin
    BeatShape[Beat].Brush.Color := LiteColor;
    BeatShape[(Beat+MaxBeat) mod (MaxBeat+1)].Brush.Color := clWhite;
    Beat := (Beat + 1) mod (MaxBeat+1);
  end;
  if( not chkMute.Checked )and( not formConfig.optNoSound.Checked ) then begin
    if( formConfig.optSystemBeep.Checked ) then begin
      Beep
    end
    else begin
      Ticker.Tick;
    end;
  end;
end;

procedure TformMain.cboBeatChange(Sender: TObject);
var
  I: integer;
begin
  Clock.Suspend;
  Beat := 0;
  for I:=0 to High(BeatShape) do
    BeatShape[I].Brush.Color := clWhite;
  Clock.Resume;
end;

procedure TformMain.btnSetupClick(Sender: TObject);
var
  ADevice, AChannel, ANote: integer;
begin
  Clock.Suspend;
  ANote := -1;
  if( Ticker <> nil ) then begin
    // Free MIDI out
    ANote := Ticker.Note;
    ADevice := Ticker.Device;
    AChannel := Ticker.Channel;
    Ticker.Free;
    Ticker := nil;
  end;
  if( formConfig.ShowModal = mrOk ) then begin
    ANote := formConfig.MidiSound;
    ADevice := formConfig.cboMidiDevice.ItemIndex;
    AChannel := formConfig.MidiChannel;
  end;
  if( ANote >= 0 ) then begin
    Ticker := TMidiTick.Create( ADevice,
      ANote,
      AChannel );
  end;
  Clock.Resume;
end;

procedure TformMain.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TformMain.shpOneMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  N, T: integer;
begin
  T := GetTickCount;
  if( Marker = 0 ) then
    Marker := T
  else begin
    N := T - Marker;
    Marker := T;
    try
      SetTempo( Round( 60 / (N / 1000) ) );
    except
    end;
  end;
end;

end.

